@@ -2,7 +2,7 @@
  * qrencode - QR Code encoder
  *
  * QR Code encoding tool
- * Copyright (C) 2006-2013 Kentaro Fukuchi <kentaro@fukuchi.org>
+ * Copyright (C) 2006-2017 Kentaro Fukuchi <kentaro@fukuchi.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,8 +25,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <png.h>
 #include <getopt.h>
+#include <errno.h>
+#if HAVE_PNG
+#include <png.h>
+#endif
 
 #include "qrencode.h"
 
@@ -40,24 +43,29 @@ static int margin = -1;
 static int dpi = 72;
 static int structured = 0;
 static int rle = 0;
+static int svg_path = 0;
 static int micro = 0;
 static QRecLevel level = QR_ECLEVEL_L;
 static QRencodeMode hint = QR_MODE_8;
-static unsigned int fg_color[4] = {0, 0, 0, 255};
-static unsigned int bg_color[4] = {255, 255, 255, 255};
+static unsigned char fg_color[4] = {0, 0, 0, 255};
+static unsigned char bg_color[4] = {255, 255, 255, 255};
 
 static int verbose = 0;
 
 enum imageType {
 	PNG_TYPE,
+	PNG32_TYPE,
 	EPS_TYPE,
 	SVG_TYPE,
+	XPM_TYPE,
 	ANSI_TYPE,
 	ANSI256_TYPE,
 	ASCII_TYPE,
 	ASCIIi_TYPE,
 	UTF8_TYPE,
-	ANSIUTF8_TYPE
+	ANSIUTF8_TYPE,
+	UTF8i_TYPE,
+	ANSIUTF8i_TYPE
 };
 
 static enum imageType image_type = PNG_TYPE;
@@ -65,6 +73,7 @@ static enum imageType image_type = PNG_TYPE;
 static const struct option options[] = {
 	{"help"         , no_argument      , NULL, 'h'},
 	{"output"       , required_argument, NULL, 'o'},
+	{"read-from"    , required_argument, NULL, 'r'},
 	{"level"        , required_argument, NULL, 'l'},
 	{"size"         , required_argument, NULL, 's'},
 	{"symversion"   , required_argument, NULL, 'v'},
@@ -77,6 +86,7 @@ static const struct option options[] = {
 	{"ignorecase"   , no_argument      , NULL, 'i'},
 	{"8bit"         , no_argument      , NULL, '8'},
 	{"rle"          , no_argument      , &rle,   1},
+	{"svg-path"     , no_argument      , &svg_path, 1},
 	{"micro"        , no_argument      , NULL, 'M'},
 	{"foreground"   , required_argument, NULL, 'f'},
 	{"background"   , required_argument, NULL, 'b'},
@@ -85,16 +95,17 @@ static const struct option options[] = {
 	{NULL, 0, NULL, 0}
 };
 
-static char *optstring = "ho:l:s:v:m:d:t:Skci8MV";
+static char *optstring = "ho:r:l:s:v:m:d:t:Skci8MV";
 
-static void usage(int help, int longopt)
+static void usage(int help, int longopt, int status)
 {
-	fprintf(stderr,
+	FILE *out = status ? stderr : stdout;
+	fprintf(out,
 "qrencode version %s\n"
-"Copyright (C) 2006-2012 Kentaro Fukuchi\n", QRcode_APIVersionString());
+"Copyright (C) 2006-2017 Kentaro Fukuchi\n", QRcode_APIVersionString());
 	if(help) {
 		if(longopt) {
-			fprintf(stderr,
+			fprintf(out,
 "Usage: qrencode [OPTION]... [STRING]\n"
 "Encode input data in a QR Code and save as a PNG or EPS image.\n\n"
 "  -h, --help   display the help message. -h displays only the help of short\n"
@@ -103,21 +114,23 @@ static void usage(int help, int longopt)
 "               write image to FILENAME. If '-' is specified, the result\n"
 "               will be output to standard output. If -S is given, structured\n"
 "               symbols are written to FILENAME-01.png, FILENAME-02.png, ...\n"
-"               (suffix is removed from FILENAME, if specified)\n"
+"               (suffix is removed from FILENAME, if specified)\n\n"
+"  -r FILENAME, --read-from=FILENAME\n"
+"               read input data from FILENAME.\n\n"
 "  -s NUMBER, --size=NUMBER\n"
 "               specify module size in dots (pixels). (default=3)\n\n"
 "  -l {LMQH}, --level={LMQH}\n"
 "               specify error correction level from L (lowest) to H (highest).\n"
 "               (default=L)\n\n"
 "  -v NUMBER, --symversion=NUMBER\n"
-"               specify the version of the symbol. See SYMBOL VERSIONS for more\n"
-"               information. (default=auto)\n\n"
+"               specify the minimum version of the symbol. See SYMBOL VERSIONS\n"
+"               for more information. (default=auto)\n\n"
 "  -m NUMBER, --margin=NUMBER\n"
 "               specify the width of the margins. (default=4 (2 for Micro QR)))\n\n"
 "  -d NUMBER, --dpi=NUMBER\n"
 "               specify the DPI of the generated PNG. (default=72)\n\n"
-"  -t {PNG,EPS,SVG,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8}, --type={PNG,EPS,\n"
-"               SVG,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8}\n"
+"  -t {PNG,PNG32,EPS,SVG,XPM,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8},\n"
+"  --type={PNG,PNG32,EPS,SVG,XPM,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8}\n"
 "               specify the type of the generated image. (default=PNG)\n\n"
 "  -S, --structured\n"
 "               make structured symbols. Version must be specified.\n\n"
@@ -128,19 +141,21 @@ static void usage(int help, int longopt)
 "               ignore case distinctions and use only upper-case characters.\n\n"
 "  -8, --8bit   encode entire data in 8-bit mode. -k, -c and -i will be ignored.\n\n"
 "      --rle    enable run-length encoding for SVG.\n\n"
+"      --svg-path\n"
+"               use single path to draw modules for SVG.\n\n"
 "  -M, --micro  encode in a Micro QR Code. (experimental)\n\n"
 "      --foreground=RRGGBB[AA]\n"
 "      --background=RRGGBB[AA]\n"
 "               specify foreground/background color in hexadecimal notation.\n"
 "               6-digit (RGB) or 8-digit (RGBA) form are supported.\n"
-"               Color output support available only in PNG and SVG.\n"
+"               Color output support available only in PNG, EPS and SVG.\n\n"
 "  -V, --version\n"
 "               display the version number and copyrights of the qrencode.\n\n"
 "      --verbose\n"
 "               display verbose information to stderr.\n\n"
 "  [STRING]     input data. If it is not specified, data will be taken from\n"
 "               standard input.\n\n"
-"*SYMBOL VERSIONS\n"
+"SYMBOL VERSIONS\n"
 "               The symbol versions of QR Code range from Version 1 to Version\n"
 "               40. Each version has a different module configuration or number\n"
 "               of modules, ranging from Version 1 (21 x 21 modules) up to\n"
@@ -151,7 +166,7 @@ static void usage(int help, int longopt)
 
 			);
 		} else {
-			fprintf(stderr,
+			fprintf(out,
 "Usage: qrencode [OPTION]... [STRING]\n"
 "Encode input data in a QR Code and save as a PNG or EPS image.\n\n"
 "  -h           display this message.\n"
@@ -160,13 +175,14 @@ static void usage(int help, int longopt)
 "               will be output to standard output. If -S is given, structured\n"
 "               symbols are written to FILENAME-01.png, FILENAME-02.png, ...\n"
 "               (suffix is removed from FILENAME, if specified)\n"
+"  -r FILENAME  read input data from FILENAME.\n"
 "  -s NUMBER    specify module size in dots (pixels). (default=3)\n"
 "  -l {LMQH}    specify error correction level from L (lowest) to H (highest).\n"
 "               (default=L)\n"
-"  -v NUMBER    specify the version of the symbol. (default=auto)\n"
+"  -v NUMBER    specify the minimum version of the symbol. (default=auto)\n"
 "  -m NUMBER    specify the width of the margins. (default=4 (2 for Micro))\n"
 "  -d NUMBER    specify the DPI of the generated PNG. (default=72)\n"
-"  -t {PNG,EPS,SVG,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8}\n"
+"  -t {PNG,PNG32,EPS,SVG,XPM,ANSI,ANSI256,ASCII,ASCIIi,UTF8,ANSIUTF8}\n"
 "               specify the type of the generated image. (default=PNG)\n"
 "  -S           make structured symbols. Version must be specified.\n"
 "  -k           assume that the input text contains kanji (shift-jis).\n"
@@ -174,65 +190,63 @@ static void usage(int help, int longopt)
 "  -i           ignore case distinctions and use only upper-case characters.\n"
 "  -8           encode entire data in 8-bit mode. -k, -c and -i will be ignored.\n"
 "  -M           encode in a Micro QR Code.\n"
-"  --foreground=RRGGBB[AA]\n"
-"  --background=RRGGBB[AA]\n"
-"               specify foreground/background color in hexadecimal notation.\n"
-"               6-digit (RGB) or 8-digit (RGBA) form are supported.\n"
-"               Color output support available only in PNG and SVG.\n"
 "  -V           display the version number and copyrights of the qrencode.\n"
 "  [STRING]     input data. If it is not specified, data will be taken from\n"
-"               standard input.\n"
+"               standard input.\n\n"
+"  Try \"qrencode --help\" for more options.\n"
 			);
 		}
 	}
 }
 
-static int color_set(unsigned int color[4], const char *value)
+static int color_set(unsigned char color[4], const char *value)
 {
 	int len = strlen(value);
-	int count;
+	int i, count;
+	unsigned int col[4];
 	if(len == 6) {
-		count = sscanf(value, "%02x%02x%02x%n", &color[0], &color[1], &color[2], &len);
+		count = sscanf(value, "%02x%02x%02x%n", &col[0], &col[1], &col[2], &len);
 		if(count < 3 || len != 6) {
 			return -1;
 		}
+		for(i = 0; i < 3; i++) {
+			color[i] = col[i];
+		}
 		color[3] = 255;
 	} else if(len == 8) {
-		count = sscanf(value, "%02x%02x%02x%02x%n", &color[0], &color[1], &color[2], &color[3], &len);
+		count = sscanf(value, "%02x%02x%02x%02x%n", &col[0], &col[1], &col[2], &col[3], &len);
 		if(count < 4 || len != 8) {
 			return -1;
 		}
+		for(i = 0; i < 4; i++) {
+			color[i] = col[i];
+		}
 	} else {
 		return -1;
 	}
 	return 0;
 }
 
-#define MAX_DATA_SIZE (7090 * 16) /* from the specification */
-static unsigned char *readStdin(int *length)
+#define MAX_DATA_SIZE (7090 * 2) /* timed by the safty factor 2 */
+static unsigned char data_buffer[MAX_DATA_SIZE];
+static unsigned char *readFile(FILE *fp, int *length)
 {
-	unsigned char *buffer;
 	int ret;
 
-	buffer = (unsigned char *)malloc(MAX_DATA_SIZE + 1);
-	if(buffer == NULL) {
-		fprintf(stderr, "Memory allocation failed.\n");
-		exit(EXIT_FAILURE);
-	}
-	ret = fread(buffer, 1, MAX_DATA_SIZE, stdin);
+	ret = fread(data_buffer, 1, MAX_DATA_SIZE, fp);
 	if(ret == 0) {
 		fprintf(stderr, "No input data.\n");
 		exit(EXIT_FAILURE);
 	}
-	if(feof(stdin) == 0) {
+	if(feof(fp) == 0) {
 		fprintf(stderr, "Input data is too large.\n");
 		exit(EXIT_FAILURE);
 	}
 
-	buffer[ret] = '\0';
+	data_buffer[ret] = '\0';
 	*length = ret;
 
-	return buffer;
+	return data_buffer;
 }
 
 static FILE *openFile(const char *outfile)
@@ -253,19 +267,39 @@ static FILE *openFile(const char *outfile)
 	return fp;
 }
 
-static int writePNG(QRcode *qrcode, const char *outfile)
+#if HAVE_PNG
+static void fillRow(unsigned char *row, int num, const unsigned char color[])
 {
+	int i;
+
+	for(i = 0; i < num; i++) {
+		memcpy(row, color, 4);
+		row += 4;
+	}
+}
+#endif
+
+static int writePNG(const QRcode *qrcode, const char *outfile, enum imageType type)
+{
+#if HAVE_PNG
 	static FILE *fp; // avoid clobbering by setjmp.
 	png_structp png_ptr;
 	png_infop info_ptr;
-	png_colorp palette;
+	png_colorp palette = NULL;
 	png_byte alpha_values[2];
 	unsigned char *row, *p, *q;
 	int x, y, xx, yy, bit;
 	int realwidth;
 
 	realwidth = (qrcode->width + margin * 2) * size;
-	row = (unsigned char *)malloc((realwidth + 7) / 8);
+	if(type == PNG_TYPE) {
+		row = (unsigned char *)malloc((realwidth + 7) / 8);
+	} else if(type == PNG32_TYPE) {
+		row = (unsigned char *)malloc(realwidth * 4);
+	} else {
+		fprintf(stderr, "Internal error.\n");
+		exit(EXIT_FAILURE);
+	}
 	if(row == NULL) {
 		fprintf(stderr, "Failed to allocate memory.\n");
 		exit(EXIT_FAILURE);
@@ -300,69 +334,110 @@ static int writePNG(QRcode *qrcode, const char *outfile)
 		exit(EXIT_FAILURE);
 	}
 
-	palette = (png_colorp) malloc(sizeof(png_color) * 2);
-	if(palette == NULL) {
-		fprintf(stderr, "Failed to allocate memory.\n");
-		exit(EXIT_FAILURE);
+	if(type == PNG_TYPE) {
+		palette = (png_colorp) malloc(sizeof(png_color) * 2);
+		if(palette == NULL) {
+			fprintf(stderr, "Failed to allocate memory.\n");
+			exit(EXIT_FAILURE);
+		}
+		palette[0].red   = fg_color[0];
+		palette[0].green = fg_color[1];
+		palette[0].blue  = fg_color[2];
+		palette[1].red   = bg_color[0];
+		palette[1].green = bg_color[1];
+		palette[1].blue  = bg_color[2];
+		alpha_values[0] = fg_color[3];
+		alpha_values[1] = bg_color[3];
+		png_set_PLTE(png_ptr, info_ptr, palette, 2);
+		png_set_tRNS(png_ptr, info_ptr, alpha_values, 2, NULL);
 	}
-	palette[0].red   = fg_color[0];
-	palette[0].green = fg_color[1];
-	palette[0].blue  = fg_color[2];
-	palette[1].red   = bg_color[0];
-	palette[1].green = bg_color[1];
-	palette[1].blue  = bg_color[2];
-	alpha_values[0] = fg_color[3];
-	alpha_values[1] = bg_color[3];
-	png_set_PLTE(png_ptr, info_ptr, palette, 2);
-	png_set_tRNS(png_ptr, info_ptr, alpha_values, 2, NULL);
 
 	png_init_io(png_ptr, fp);
-	png_set_IHDR(png_ptr, info_ptr,
-			realwidth, realwidth,
-			1,
-			PNG_COLOR_TYPE_PALETTE,
-			PNG_INTERLACE_NONE,
-			PNG_COMPRESSION_TYPE_DEFAULT,
-			PNG_FILTER_TYPE_DEFAULT);
+	if(type == PNG_TYPE) {
+		png_set_IHDR(png_ptr, info_ptr,
+				realwidth, realwidth,
+				1,
+				PNG_COLOR_TYPE_PALETTE,
+				PNG_INTERLACE_NONE,
+				PNG_COMPRESSION_TYPE_DEFAULT,
+				PNG_FILTER_TYPE_DEFAULT);
+	} else {
+		png_set_IHDR(png_ptr, info_ptr,
+				realwidth, realwidth,
+				8,
+				PNG_COLOR_TYPE_RGB_ALPHA,
+				PNG_INTERLACE_NONE,
+				PNG_COMPRESSION_TYPE_DEFAULT,
+				PNG_FILTER_TYPE_DEFAULT);
+	}
 	png_set_pHYs(png_ptr, info_ptr,
 			dpi * INCHES_PER_METER,
 			dpi * INCHES_PER_METER,
 			PNG_RESOLUTION_METER);
 	png_write_info(png_ptr, info_ptr);
 
+	if(type == PNG_TYPE) {
 	/* top margin */
-	memset(row, 0xff, (realwidth + 7) / 8);
-	for(y=0; y<margin * size; y++) {
-		png_write_row(png_ptr, row);
-	}
-
-	/* data */
-	p = qrcode->data;
-	for(y=0; y<qrcode->width; y++) {
-		bit = 7;
 		memset(row, 0xff, (realwidth + 7) / 8);
-		q = row;
-		q += margin * size / 8;
-		bit = 7 - (margin * size % 8);
-		for(x=0; x<qrcode->width; x++) {
-			for(xx=0; xx<size; xx++) {
-				*q ^= (*p & 1) << bit;
-				bit--;
-				if(bit < 0) {
-					q++;
-					bit = 7;
+		for(y = 0; y < margin * size; y++) {
+			png_write_row(png_ptr, row);
+		}
+
+		/* data */
+		p = qrcode->data;
+		for(y = 0; y < qrcode->width; y++) {
+			memset(row, 0xff, (realwidth + 7) / 8);
+			q = row;
+			q += margin * size / 8;
+			bit = 7 - (margin * size % 8);
+			for(x = 0; x < qrcode->width; x++) {
+				for(xx = 0; xx < size; xx++) {
+					*q ^= (*p & 1) << bit;
+					bit--;
+					if(bit < 0) {
+						q++;
+						bit = 7;
+					}
+				}
+				p++;
+			}
+			for(yy = 0; yy < size; yy++) {
+				png_write_row(png_ptr, row);
+			}
+		}
+		/* bottom margin */
+		memset(row, 0xff, (realwidth + 7) / 8);
+		for(y = 0; y < margin * size; y++) {
+			png_write_row(png_ptr, row);
+		}
+	} else {
+	/* top margin */
+		fillRow(row, realwidth, bg_color);
+		for(y = 0; y < margin * size; y++) {
+			png_write_row(png_ptr, row);
+		}
+
+		/* data */
+		p = qrcode->data;
+		for(y = 0; y < qrcode->width; y++) {
+			fillRow(row, realwidth, bg_color);
+			for(x = 0; x < qrcode->width; x++) {
+				for(xx = 0; xx < size; xx++) {
+					if(*p & 1) {
+						memcpy(&row[((margin + x) * size + xx) * 4], fg_color, 4);
+					}
 				}
+				p++;
+			}
+			for(yy = 0; yy < size; yy++) {
+				png_write_row(png_ptr, row);
 			}
-			p++;
 		}
-		for(yy=0; yy<size; yy++) {
+		/* bottom margin */
+		fillRow(row, realwidth, bg_color);
+		for(y = 0; y < margin * size; y++) {
 			png_write_row(png_ptr, row);
 		}
-	}
-	/* bottom margin */
-	memset(row, 0xff, (realwidth + 7) / 8);
-	for(y=0; y<margin * size; y++) {
-		png_write_row(png_ptr, row);
 	}
 
 	png_write_end(png_ptr, info_ptr);
@@ -373,9 +448,13 @@ static int writePNG(QRcode *qrcode, const char *outfile)
 	free(palette);
 
 	return 0;
+#else
+	fputs("PNG output is disabled at compile time. No output generated.\n", stderr);
+	return 0;
+#endif
 }
 
-static int writeEPS(QRcode *qrcode, const char *outfile)
+static int writeEPS(const QRcode *qrcode, const char *outfile)
 {
 	FILE *fp;
 	unsigned char *row, *p;
@@ -383,7 +462,7 @@ static int writeEPS(QRcode *qrcode, const char *outfile)
 	int realwidth;
 
 	fp = openFile(outfile);
-   
+
 	realwidth = (qrcode->width + margin * 2) * size;
 	/* EPS file header */
 	fprintf(fp, "%%!PS-Adobe-2.0 EPSF-1.2\n"
@@ -397,16 +476,28 @@ static int writeEPS(QRcode *qrcode, const char *outfile)
 				"1 0 rlineto "
 				"0 -1 rlineto "
 				"fill "
-				"} bind def "
-				"%d %d scale ", size, size);
-	
+				"} bind def\n");
+	/* set color */
+	fprintf(fp, "gsave\n");
+	fprintf(fp, "%f %f %f setrgbcolor\n",
+			(float)bg_color[0] / 255,
+			(float)bg_color[1] / 255,
+			(float)bg_color[2] / 255);
+	fprintf(fp, "%d %d scale\n", realwidth, realwidth);
+	fprintf(fp, "0 0 p\ngrestore\n");
+	fprintf(fp, "%f %f %f setrgbcolor\n",
+			(float)fg_color[0] / 255,
+			(float)fg_color[1] / 255,
+			(float)fg_color[2] / 255);
+	fprintf(fp, "%d %d scale\n", size, size);
+
 	/* data */
 	p = qrcode->data;
-	for(y=0; y<qrcode->width; y++) {
+	for(y = 0; y < qrcode->width; y++) {
 		row = (p+(y*qrcode->width));
 		yy = (margin + qrcode->width - y - 1);
-		
-		for(x=0; x<qrcode->width; x++) {
+
+		for(x = 0; x < qrcode->width; x++) {
 			if(*(row+x)&0x1) {
 				fprintf(fp, "%d %d p ", margin + x,  yy);
 			}
@@ -419,20 +510,24 @@ static int writeEPS(QRcode *qrcode, const char *outfile)
 	return 0;
 }
 
-static void writeSVG_writeRect(FILE *fp, int x, int y, int width, char* col, float opacity)
+static void writeSVG_drawModules(FILE *fp, int x, int y, int width, const char* col, float opacity)
 {
-	if(fg_color[3] != 255) {
-		fprintf(fp, "\t\t\t<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"1\" "\
-				"fill=\"#%s\" fill-opacity=\"%f\" />\n", 
-				x, y, width, col, opacity );
+	if(svg_path) {
+		fprintf(fp, "M%d,%dh%d", x, y, width);
 	} else {
-		fprintf(fp, "\t\t\t<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"1\" "\
-				"fill=\"#%s\" />\n", 
-				x, y, width, col );
+		if(fg_color[3] != 255) {
+			fprintf(fp, "\t\t\t<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"1\" "\
+					"fill=\"#%s\" fill-opacity=\"%f\"/>\n",
+					x, y, width, col, opacity );
+		} else {
+			fprintf(fp, "\t\t\t<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"1\" "\
+					"fill=\"#%s\"/>\n",
+					x, y, width, col );
+		}
 	}
 }
 
-static int writeSVG( QRcode *qrcode, const char *outfile )
+static int writeSVG(const QRcode *qrcode, const char *outfile)
 {
 	FILE *fp;
 	unsigned char *row, *p;
@@ -458,101 +553,189 @@ static int writeSVG( QRcode *qrcode, const char *outfile )
 	/* XML declaration */
 	fputs( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n", fp );
 
-	/* DTD 
-	   No document type specified because "while a DTD is provided in [the SVG] 
-	   specification, the use of DTDs for validating XML documents is known to be 
-	   problematic. In particular, DTDs do not handle namespaces gracefully. It 
-	   is *not* recommended that a DOCTYPE declaration be included in SVG 
-	   documents." 
+	/* DTD
+	   No document type specified because "while a DTD is provided in [the SVG]
+	   specification, the use of DTDs for validating XML documents is known to
+	   be problematic. In particular, DTDs do not handle namespaces gracefully.
+	   It is *not* recommended that a DOCTYPE declaration be included in SVG
+	   documents."
 	   http://www.w3.org/TR/2003/REC-SVG11-20030114/intro.html#Namespace
 	*/
 
 	/* Vanity remark */
-	fprintf( fp, "<!-- Created with qrencode %s (http://fukuchi.org/works/qrencode/index.html.en) -->\n", 
-			QRcode_APIVersionString() );
+	fprintf(fp, "<!-- Created with qrencode %s (https://fukuchi.org/works/qrencode/index.html) -->\n", QRcode_APIVersionString());
 
 	/* SVG code start */
-	fprintf( fp, "<svg width=\"%0.2fcm\" height=\"%0.2fcm\" viewBox=\"0 0 %d %d\""\
+	fprintf(fp,
+			"<svg width=\"%.2fcm\" height=\"%.2fcm\" viewBox=\"0 0 %d %d\""\
 			" preserveAspectRatio=\"none\" version=\"1.1\""\
-			" xmlns=\"http://www.w3.org/2000/svg\">\n", 
+			" xmlns=\"http://www.w3.org/2000/svg\">\n",
 			realwidth / scale, realwidth / scale, symwidth, symwidth
 		   );
 
 	/* Make named group */
-	fputs( "\t<g id=\"QRcode\">\n", fp );
+	fputs("\t<g id=\"QRcode\">\n", fp);
 
 	/* Make solid background */
 	if(bg_color[3] != 255) {
-		fprintf(fp, "\t\t<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" fill=\"#%s\" fill-opacity=\"%f\" />\n", symwidth, symwidth, bg, bg_opacity);
+		fprintf(fp, "\t\t<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" fill=\"#%s\" fill-opacity=\"%f\"/>\n", symwidth, symwidth, bg, bg_opacity);
 	} else {
-		fprintf(fp, "\t\t<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" fill=\"#%s\" />\n", symwidth, symwidth, bg);
+		fprintf(fp, "\t\t<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" fill=\"#%s\"/>\n", symwidth, symwidth, bg);
 	}
 
-	/* Create new viewbox for QR data */
-	fputs( "\t\t<g id=\"Pattern\">\n", fp);
+	if(svg_path) {
+		if(fg_color[3] != 255) {
+			fprintf(fp, "\t\t<path style=\"stroke:#%s;stroke-opacity:%f\" transform=\"translate(%d,%d.5)\" d=\"", fg, fg_opacity, margin, margin);
+		} else {
+			fprintf(fp, "\t\t<path style=\"stroke:#%s\" transform=\"translate(%d,%d.5)\" d=\"", fg, margin, margin);
+		}
+	} else {
+		/* Create new viewbox for QR data */
+		fprintf(fp, "\t\t<g id=\"Pattern\" transform=\"translate(%d,%d)\">\n", margin, margin);
+	}
 
 	/* Write data */
 	p = qrcode->data;
-	for(y=0; y<qrcode->width; y++) {
+	for(y = 0; y < qrcode->width; y++) {
 		row = (p+(y*qrcode->width));
 
 		if( !rle ) {
 			/* no RLE */
-			for(x=0; x<qrcode->width; x++) {
+			for(x = 0; x < qrcode->width; x++) {
 				if(*(row+x)&0x1) {
-					writeSVG_writeRect(fp,	margin + x,
-								margin + y, 1,
-								fg, fg_opacity);
+					writeSVG_drawModules(fp, x, y, 1, fg, fg_opacity);
 				}
 			}
 		} else {
 			/* simple RLE */
 			pen = 0;
 			x0  = 0;
-			for(x=0; x<qrcode->width; x++) {
+			for(x = 0; x < qrcode->width; x++) {
 				if( !pen ) {
 					pen = *(row+x)&0x1;
 					x0 = x;
-				} else {
-					if(!(*(row+x)&0x1)) {
-						writeSVG_writeRect(fp, x0 + margin, y + margin, x-x0, fg, fg_opacity);
-						pen = 0;
-					}
+				} else if(!(*(row+x)&0x1)) {
+					writeSVG_drawModules(fp, x0, y, x-x0, fg, fg_opacity);
+					pen = 0;
 				}
 			}
 			if( pen ) {
-				writeSVG_writeRect(fp, x0 + margin, y + margin, qrcode->width - x0, fg, fg_opacity);
+				writeSVG_drawModules(fp, x0, y, qrcode->width - x0, fg, fg_opacity);
 			}
 		}
 	}
-	/* Close QR data viewbox */
-	fputs( "\t\t</g>\n", fp );
+
+	if(svg_path) {
+		fputs("\"/>\n", fp);
+	} else {
+		/* Close QR data viewbox */
+		fputs("\t\t</g>\n", fp);
+	}
 
 	/* Close group */
-	fputs( "\t</g>\n", fp );
+	fputs("\t</g>\n", fp);
 
 	/* Close SVG code */
-	fputs( "</svg>\n", fp );
-	fclose( fp );
+	fputs("</svg>\n", fp);
+	fclose(fp);
+
+	return 0;
+}
+
+static int writeXPM(const QRcode *qrcode, const char *outfile)
+{
+	FILE *fp;
+	int x, xx, y, yy, realwidth, realmargin;
+	char *row;
+	char fg[7], bg[7];
+	unsigned char *p;
+
+	fp = openFile(outfile);
+
+	realwidth = (qrcode->width + margin * 2) * size;
+	realmargin = margin * size;
+
+	row = malloc(realwidth + 1);
+	if (!row ) {
+		fprintf(stderr, "Failed to allocate memory.\n");
+		exit(EXIT_FAILURE);
+	}
+
+	snprintf(fg, 7, "%02x%02x%02x", fg_color[0], fg_color[1],  fg_color[2]);
+	snprintf(bg, 7, "%02x%02x%02x", bg_color[0], bg_color[1],  bg_color[2]);
+
+	fputs("/* XPM */\n", fp);
+	fputs("static const char *const qrcode_xpm[] = {\n", fp);
+	fputs("/* width height ncolors chars_per_pixel */\n", fp);
+	fprintf(fp, "\"%d %d 2 1\",\n", realwidth, realwidth);
+
+	fputs("/* colors */\n", fp);
+	fprintf(fp, "\"F c #%s\",\n", fg);
+	fprintf(fp, "\"B c #%s\",\n", bg);
+
+	fputs("/* pixels */\n", fp);
+	memset(row, 'B', realwidth);
+	row[realwidth] = '\0';
+
+	for (y = 0; y < realmargin; y++) {
+		fprintf(fp, "\"%s\",\n", row);
+	}
+
+	p = qrcode->data;
+	for (y = 0; y < qrcode->width; y++) {
+		for (yy = 0; yy < size; yy++) {
+			fputs("\"", fp);
+
+			for (x = 0; x < margin; x++) {
+				for (xx = 0; xx < size; xx++) {
+					fputs("B", fp);
+				}
+			}
+
+			for (x = 0; x < qrcode->width; x++) {
+				for (xx = 0; xx < size; xx++) {
+					if (p[(y * qrcode->width) + x] & 0x1) {
+						fputs("F", fp);
+					} else {
+						fputs("B", fp);
+					}
+				}
+			}
+
+			for (x = 0; x < margin; x++) {
+				for (xx = 0; xx < size; xx++) {
+					fputs("B", fp);
+				}
+			}
+
+			fputs("\",\n", fp);
+		}
+	}
+
+	for (y = 0; y < realmargin; y++) {
+		fprintf(fp, "\"%s\"%s\n", row, y < (size - 1) ? "," : "};");
+	}
+
+	free(row);
+	fclose(fp);
 
 	return 0;
 }
 
 static void writeANSI_margin(FILE* fp, int realwidth,
-                             char* buffer, int buffer_s,
-                             char* white, int white_s )
+                             char* buffer, const char* white, int white_s )
 {
 	int y;
 
 	strncpy(buffer, white, white_s);
 	memset(buffer + white_s, ' ', realwidth * 2);
 	strcpy(buffer + white_s + realwidth * 2, "\033[0m\n"); // reset to default colors
-	for(y=0; y<margin; y++ ){
+	for(y = 0; y < margin; y++ ){
 		fputs(buffer, fp);
 	}
 }
 
-static int writeANSI(QRcode *qrcode, const char *outfile)
+static int writeANSI(const QRcode *qrcode, const char *outfile)
 {
 	FILE *fp;
 	unsigned char *row, *p;
@@ -560,10 +743,11 @@ static int writeANSI(QRcode *qrcode, const char *outfile)
 	int realwidth;
 	int last;
 
-	char *white, *black, *buffer;
+	const char *white, *black;
+	char *buffer;
 	int white_s, black_s, buffer_s;
 
-	if( image_type == ANSI256_TYPE ){
+	if(image_type == ANSI256_TYPE){
 		/* codes for 256 color compatible terminals */
 		white = "\033[48;5;231m";
 		white_s = 11;
@@ -581,55 +765,53 @@ static int writeANSI(QRcode *qrcode, const char *outfile)
 	fp = openFile(outfile);
 
 	realwidth = (qrcode->width + margin * 2) * size;
-	buffer_s = ( realwidth * white_s ) * 2;
-	buffer = (char *)malloc( buffer_s );
+	buffer_s = (realwidth * white_s) * 2;
+	buffer = (char *)malloc(buffer_s);
 	if(buffer == NULL) {
 		fprintf(stderr, "Failed to allocate memory.\n");
 		exit(EXIT_FAILURE);
 	}
 
 	/* top margin */
-	writeANSI_margin(fp, realwidth, buffer, buffer_s, white, white_s);
+	writeANSI_margin(fp, realwidth, buffer, white, white_s);
 
 	/* data */
 	p = qrcode->data;
-	for(y=0; y<qrcode->width; y++) {
+	for(y = 0; y < qrcode->width; y++) {
 		row = (p+(y*qrcode->width));
 
-		bzero( buffer, buffer_s );
-		strncpy( buffer, white, white_s );
-		for(x=0; x<margin; x++ ){
-			strncat( buffer, "  ", 2 );
+		memset(buffer, 0, buffer_s);
+		strncpy(buffer, white, white_s);
+		for(x = 0; x < margin; x++ ){
+			strncat(buffer, "  ", 2);
 		}
 		last = 0;
 
-		for(x=0; x<qrcode->width; x++) {
+		for(x = 0; x < qrcode->width; x++) {
 			if(*(row+x)&0x1) {
 				if( last != 1 ){
-					strncat( buffer, black, black_s );
+					strncat(buffer, black, black_s);
 					last = 1;
 				}
-			} else {
-				if( last != 0 ){
-					strncat( buffer, white, white_s );
-					last = 0;
-				}
+			} else if( last != 0 ){
+				strncat(buffer, white, white_s);
+				last = 0;
 			}
-			strncat( buffer, "  ", 2 );
+			strncat(buffer, "  ", 2);
 		}
 
 		if( last != 0 ){
-			strncat( buffer, white, white_s );
+			strncat(buffer, white, white_s);
 		}
-		for(x=0; x<margin; x++ ){
-			strncat( buffer, "  ", 2 );
+		for(x = 0; x < margin; x++ ){
+			strncat(buffer, "  ", 2);
 		}
-		strncat( buffer, "\033[0m\n", 5 );
-		fputs( buffer, fp );
+		strncat(buffer, "\033[0m\n", 5);
+		fputs(buffer, fp);
 	}
 
 	/* bottom margin */
-	writeANSI_margin(fp, realwidth, buffer, buffer_s, white, white_s);
+	writeANSI_margin(fp, realwidth, buffer, white, white_s);
 
 	fclose(fp);
 	free(buffer);
@@ -637,27 +819,44 @@ static int writeANSI(QRcode *qrcode, const char *outfile)
 	return 0;
 }
 
-static void writeUTF8_margin(FILE* fp, int realwidth,
-			     const char* white, const char *reset,
-			     int use_ansi)
+static void writeUTF8_margin(FILE* fp, int realwidth, const char* white,
+                             const char *reset, const char* full)
 {
 	int x, y;
 
 	for (y = 0; y < margin/2; y++) {
 		fputs(white, fp);
 		for (x = 0; x < realwidth; x++)
-			fputs("\342\226\210", fp);
+			fputs(full, fp);
 		fputs(reset, fp);
 		fputc('\n', fp);
 	}
 }
 
-static int writeUTF8(QRcode *qrcode, const char *outfile, int use_ansi)
+static int writeUTF8(const QRcode *qrcode, const char *outfile, int use_ansi, int invert)
 {
 	FILE *fp;
 	int x, y;
 	int realwidth;
 	const char *white, *reset;
+	const char *empty, *lowhalf, *uphalf, *full;
+
+	empty = " ";
+	lowhalf = "\342\226\204";
+	uphalf = "\342\226\200";
+	full = "\342\226\210";
+
+	if (invert) {
+		const char *tmp;
+
+		tmp = empty;
+		empty = full;
+		full = tmp;
+
+		tmp = lowhalf;
+		lowhalf = uphalf;
+		uphalf = tmp;
+	}
 
 	if (use_ansi){
 		white = "\033[40;37;1m";
@@ -672,7 +871,7 @@ static int writeUTF8(QRcode *qrcode, const char *outfile, int use_ansi)
 	realwidth = (qrcode->width + margin * 2);
 
 	/* top margin */
-	writeUTF8_margin(fp, realwidth, white, reset, use_ansi);
+	writeUTF8_margin(fp, realwidth, white, reset, full);
 
 	/* data */
 	for(y = 0; y < qrcode->width; y += 2) {
@@ -682,41 +881,40 @@ static int writeUTF8(QRcode *qrcode, const char *outfile, int use_ansi)
 
 		fputs(white, fp);
 
-		for (x = 0; x < margin; x++)
-			fputs("\342\226\210", fp);
+		for (x = 0; x < margin; x++) {
+			fputs(full, fp);
+		}
 
 		for (x = 0; x < qrcode->width; x++) {
 			if(row1[x] & 1) {
 				if(y < qrcode->width - 1 && row2[x] & 1) {
-					fputc(' ', fp);
+					fputs(empty, fp);
 				} else {
-					fputs("\342\226\204", fp);
+					fputs(lowhalf, fp);
 				}
+			} else if(y < qrcode->width - 1 && row2[x] & 1) {
+				fputs(uphalf, fp);
 			} else {
-				if(y < qrcode->width - 1 && row2[x] & 1) {
-					fputs("\342\226\200", fp);
-				} else {
-					fputs("\342\226\210", fp);
-				}
+				fputs(full, fp);
 			}
 		}
 
 		for (x = 0; x < margin; x++)
-			fputs("\342\226\210", fp);
+			fputs(full, fp);
 
 		fputs(reset, fp);
 		fputc('\n', fp);
 	}
 
 	/* bottom margin */
-	writeUTF8_margin(fp, realwidth, white, reset, use_ansi);
+	writeUTF8_margin(fp, realwidth, white, reset, full);
 
 	fclose(fp);
 
 	return 0;
 }
 
-static void writeASCII_margin(FILE* fp, int realwidth, char* buffer, int buffer_s, int invert)
+static void writeASCII_margin(FILE* fp, int realwidth, char* buffer, int invert)
 {
 	int y, h;
 
@@ -725,12 +923,12 @@ static void writeASCII_margin(FILE* fp, int realwidth, char* buffer, int buffer_
 	memset(buffer, (invert?'#':' '), realwidth);
 	buffer[realwidth] = '\n';
 	buffer[realwidth + 1] = '\0';
-	for(y=0; y<h; y++ ){
+	for(y = 0; y < h; y++ ){
 		fputs(buffer, fp);
 	}
 }
 
-static int writeASCII(QRcode *qrcode, const char *outfile, int invert)
+static int writeASCII(const QRcode *qrcode, const char *outfile, int invert)
 {
 	FILE *fp;
 	unsigned char *row;
@@ -759,17 +957,17 @@ static int writeASCII(QRcode *qrcode, const char *outfile, int invert)
 	}
 
 	/* top margin */
-	writeASCII_margin(fp, realwidth, buffer, buffer_s, invert);
+	writeASCII_margin(fp, realwidth, buffer, invert);
 
 	/* data */
-	for(y=0; y<qrcode->width; y++) {
+	for(y = 0; y < qrcode->width; y++) {
 		row = qrcode->data+(y*qrcode->width);
 		p = buffer;
 
 		memset(p, white, margin * 2);
 		p += margin * 2;
 
-		for(x=0; x<qrcode->width; x++) {
+		for(x = 0; x < qrcode->width; x++) {
 			if(row[x]&0x1) {
 				*p++ = black;
 				*p++ = black;
@@ -787,7 +985,7 @@ static int writeASCII(QRcode *qrcode, const char *outfile, int invert)
 	}
 
 	/* bottom margin */
-	writeASCII_margin(fp, realwidth, buffer, buffer_s, invert);
+	writeASCII_margin(fp, realwidth, buffer, invert);
 
 	fclose(fp);
 	free(buffer);
@@ -805,12 +1003,10 @@ static QRcode *encode(const unsigned char *intext, int length)
 		} else {
 			code = QRcode_encodeStringMQR((char *)intext, version, level, hint, casesensitive);
 		}
+	} else if(eightbit) {
+		code = QRcode_encodeData(length, intext, version, level);
 	} else {
-		if(eightbit) {
-			code = QRcode_encodeData(length, intext, version, level);
-		} else {
-			code = QRcode_encodeString((char *)intext, version, level, hint, casesensitive);
-		}
+		code = QRcode_encodeString((char *)intext, version, level, hint, casesensitive);
 	}
 
 	return code;
@@ -819,10 +1015,14 @@ static QRcode *encode(const unsigned char *intext, int length)
 static void qrencode(const unsigned char *intext, int length, const char *outfile)
 {
 	QRcode *qrcode;
-	
+
 	qrcode = encode(intext, length);
 	if(qrcode == NULL) {
-		perror("Failed to encode the input data");
+		if(errno == ERANGE) {
+			fprintf(stderr, "Failed to encode the input data: Input data too large\n");
+		} else {
+			perror("Failed to encode the input data");
+		}
 		exit(EXIT_FAILURE);
 	}
 
@@ -832,7 +1032,8 @@ static void qrencode(const unsigned char *intext, int length, const char *outfil
 
 	switch(image_type) {
 		case PNG_TYPE:
-			writePNG(qrcode, outfile);
+		case PNG32_TYPE:
+			writePNG(qrcode, outfile, image_type);
 			break;
 		case EPS_TYPE:
 			writeEPS(qrcode, outfile);
@@ -840,6 +1041,9 @@ static void qrencode(const unsigned char *intext, int length, const char *outfil
 		case SVG_TYPE:
 			writeSVG(qrcode, outfile);
 			break;
+		case XPM_TYPE:
+			writeXPM(qrcode, outfile);
+			break;
 		case ANSI_TYPE:
 		case ANSI256_TYPE:
 			writeANSI(qrcode, outfile);
@@ -851,10 +1055,16 @@ static void qrencode(const unsigned char *intext, int length, const char *outfil
 			writeASCII(qrcode, outfile,  0);
 			break;
 		case UTF8_TYPE:
-			writeUTF8(qrcode, outfile, 0);
+			writeUTF8(qrcode, outfile, 0, 0);
 			break;
 		case ANSIUTF8_TYPE:
-			writeUTF8(qrcode, outfile, 1);
+			writeUTF8(qrcode, outfile, 1, 0);
+			break;
+		case UTF8i_TYPE:
+			writeUTF8(qrcode, outfile, 0, 1);
+			break;
+		case ANSIUTF8i_TYPE:
+			writeUTF8(qrcode, outfile, 1, 1);
 			break;
 		default:
 			fprintf(stderr, "Unknown image type.\n");
@@ -896,11 +1106,16 @@ static void qrencodeStructured(const unsigned char *intext, int length, const ch
 		case SVG_TYPE:
 			type_suffix = ".svg";
 			break;
+		case XPM_TYPE:
+			type_suffix = ".xpm";
+			break;
 		case ANSI_TYPE:
 		case ANSI256_TYPE:
 		case ASCII_TYPE:
 		case UTF8_TYPE:
 		case ANSIUTF8_TYPE:
+		case UTF8i_TYPE:
+		case ANSIUTF8i_TYPE:
 			type_suffix = ".txt";
 			break;
 		default:
@@ -925,10 +1140,14 @@ static void qrencodeStructured(const unsigned char *intext, int length, const ch
 			*q = '\0';
 		}
 	}
-	
+
 	qrlist = encodeStructured(intext, length);
 	if(qrlist == NULL) {
-		perror("Failed to encode the input data");
+		if(errno == ERANGE) {
+			fprintf(stderr, "Failed to encode the input data: Input data too large\n");
+		} else {
+			perror("Failed to encode the input data");
+		}
 		exit(EXIT_FAILURE);
 	}
 
@@ -948,15 +1167,19 @@ static void qrencodeStructured(const unsigned char *intext, int length, const ch
 		}
 
 		switch(image_type) {
-			case PNG_TYPE: 
-				writePNG(p->code, filename);
+			case PNG_TYPE:
+			case PNG32_TYPE:
+				writePNG(p->code, filename, image_type);
 				break;
-			case EPS_TYPE: 
+			case EPS_TYPE:
 				writeEPS(p->code, filename);
 				break;
-			case SVG_TYPE: 
+			case SVG_TYPE:
 				writeSVG(p->code, filename);
 				break;
+			case XPM_TYPE:
+				writeXPM(p->code, filename);
+				break;
 			case ANSI_TYPE:
 			case ANSI256_TYPE:
 				writeANSI(p->code, filename);
@@ -968,10 +1191,16 @@ static void qrencodeStructured(const unsigned char *intext, int length, const ch
 				writeASCII(p->code, filename, 0);
 				break;
 			case UTF8_TYPE:
-				writeUTF8(p->code, filename, 0);
+				writeUTF8(p->code, filename, 0, 0);
 				break;
 			case ANSIUTF8_TYPE:
-				writeUTF8(p->code, filename, 0);
+				writeUTF8(p->code, filename, 0, 0);
+				break;
+			case UTF8i_TYPE:
+				writeUTF8(p->code, filename, 0, 1);
+				break;
+			case ANSIUTF8i_TYPE:
+				writeUTF8(p->code, filename, 0, 1);
 				break;
 
 			default:
@@ -992,23 +1221,26 @@ static void qrencodeStructured(const unsigned char *intext, int length, const ch
 int main(int argc, char **argv)
 {
 	int opt, lindex = -1;
-	char *outfile = NULL;
+	char *outfile = NULL, *infile = NULL;
 	unsigned char *intext = NULL;
 	int length = 0;
+	FILE *fp;
 
 	while((opt = getopt_long(argc, argv, optstring, options, &lindex)) != -1) {
 		switch(opt) {
 			case 'h':
 				if(lindex == 0) {
-					usage(1, 1);
+					usage(1, 1, EXIT_SUCCESS);
 				} else {
-					usage(1, 0);
+					usage(1, 0, EXIT_SUCCESS);
 				}
 				exit(EXIT_SUCCESS);
-				break;
 			case 'o':
 				outfile = optarg;
 				break;
+			case 'r':
+				infile = optarg;
+				break;
 			case 's':
 				size = atoi(optarg);
 				if(size <= 0) {
@@ -1044,7 +1276,6 @@ int main(int argc, char **argv)
 					default:
 						fprintf(stderr, "Invalid level: %s\n", optarg);
 						exit(EXIT_FAILURE);
-						break;
 				}
 				break;
 			case 'm':
@@ -1062,12 +1293,16 @@ int main(int argc, char **argv)
 				}
 				break;
 			case 't':
-				if(strcasecmp(optarg, "png") == 0) {
+				if(strcasecmp(optarg, "png32") == 0) {
+					image_type = PNG32_TYPE;
+				} else if(strcasecmp(optarg, "png") == 0) {
 					image_type = PNG_TYPE;
 				} else if(strcasecmp(optarg, "eps") == 0) {
 					image_type = EPS_TYPE;
 				} else if(strcasecmp(optarg, "svg") == 0) {
 					image_type = SVG_TYPE;
+				} else if(strcasecmp(optarg, "xpm") == 0) {
+					image_type = XPM_TYPE;
 				} else if(strcasecmp(optarg, "ansi") == 0) {
 					image_type = ANSI_TYPE;
 				} else if(strcasecmp(optarg, "ansi256") == 0) {
@@ -1080,6 +1315,10 @@ int main(int argc, char **argv)
 					image_type = UTF8_TYPE;
 				} else if(strcasecmp(optarg, "ansiutf8") == 0) {
 					image_type = ANSIUTF8_TYPE;
+				} else if(strcasecmp(optarg, "utf8i") == 0) {
+					image_type = UTF8i_TYPE;
+				} else if(strcasecmp(optarg, "ansiutf8i") == 0) {
+					image_type = ANSIUTF8i_TYPE;
 				} else {
 					fprintf(stderr, "Invalid image type: %s\n", optarg);
 					exit(EXIT_FAILURE);
@@ -1116,21 +1355,19 @@ int main(int argc, char **argv)
 				}
 				break;
 			case 'V':
-				usage(0, 0);
+				usage(0, 0, EXIT_SUCCESS);
 				exit(EXIT_SUCCESS);
-				break;
 			case 0:
 				break;
 			default:
-				fprintf(stderr, "Try `qrencode --help' for more information.\n");
+				fprintf(stderr, "Try \"qrencode --help\" for more information.\n");
 				exit(EXIT_FAILURE);
-				break;
 		}
 	}
 
 	if(argc == 1) {
-		usage(1, 0);
-		exit(EXIT_SUCCESS);
+		usage(1, 0, EXIT_FAILURE);
+		exit(EXIT_FAILURE);
 	}
 
 	if(outfile == NULL && image_type == PNG_TYPE) {
@@ -1143,7 +1380,13 @@ int main(int argc, char **argv)
 		length = strlen((char *)intext);
 	}
 	if(intext == NULL) {
-		intext = readStdin(&length);
+		fp = infile == NULL ? stdin : fopen(infile,"r");
+		if(fp == 0) {
+			fprintf(stderr, "Can not read input file %s.\n", infile);
+			exit(EXIT_FAILURE);
+		}
+		intext = readFile(fp,&length);
+
 	}
 
 	if(micro && version > MQRSPEC_VERSION_MAX) {
