@@ -2,7 +2,7 @@
  * qrencode - QR Code encoder
  *
  * Input data chunk class
- * Copyright (C) 2006-2011 Kentaro Fukuchi <kentaro@fukuchi.org>
+ * Copyright (C) 2006-2017 Kentaro Fukuchi <kentaro@fukuchi.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -59,6 +59,7 @@ static QRinput_List *QRinput_List_newEntry(QRencodeMode mode, int size, const un
 
 	entry->mode = mode;
 	entry->size = size;
+	entry->data = NULL;
 	if(size > 0) {
 		entry->data = (unsigned char *)malloc(size);
 		if(entry->data == NULL) {
@@ -244,7 +245,7 @@ int QRinput_append(QRinput *input, QRencodeMode mode, int size, const unsigned c
  * @throw EINVAL invalid parameter.
  * @throw ENOMEM unable to allocate memory.
  */
-__STATIC int QRinput_insertStructuredAppendHeader(QRinput *input, int size, int number, unsigned char parity)
+STATIC_IN_RELEASE int QRinput_insertStructuredAppendHeader(QRinput *input, int size, int number, unsigned char parity)
 {
 	QRinput_List *entry;
 	unsigned char buf[3];
@@ -314,7 +315,7 @@ static unsigned char QRinput_calcParity(QRinput *input)
 	list = input->head;
 	while(list != NULL) {
 		if(list->mode != QR_MODE_STRUCTURE) {
-			for(i=list->size-1; i>=0; i--) {
+			for(i = list->size-1; i >= 0; i--) {
 				parity ^= list->data[i];
 			}
 		}
@@ -364,7 +365,7 @@ static int QRinput_checkModeNum(int size, const char *data)
 {
 	int i;
 
-	for(i=0; i<size; i++) {
+	for(i = 0; i < size; i++) {
 		if(data[i] < '0' || data[i] > '9')
 			return -1;
 	}
@@ -373,7 +374,7 @@ static int QRinput_checkModeNum(int size, const char *data)
 }
 
 /**
- * Estimates the length of the encoded bit stream of numeric data.
+ * Estimate the length of the encoded bit stream of numeric data.
  * @param size
  * @return number of bits
  */
@@ -399,7 +400,7 @@ int QRinput_estimateBitsModeNum(int size)
 }
 
 /**
- * Convert the number data to a bit stream.
+ * Convert the number data and append to a bit stream.
  * @param entry
  * @param mqr
  * @retval 0 success
@@ -407,55 +408,48 @@ int QRinput_estimateBitsModeNum(int size)
  *            See Execptions for the details.
  * @throw ENOMEM unable to allocate memory.
  */
-static int QRinput_encodeModeNum(QRinput_List *entry, int version, int mqr)
+static int QRinput_encodeModeNum(QRinput_List *entry, BitStream *bstream, int version, int mqr)
 {
 	int words, i, ret;
 	unsigned int val;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
-
 	if(mqr) {
 		if(version > 1) {
-			ret = BitStream_appendNum(entry->bstream, version - 1, MQRSPEC_MODEID_NUM);
-			if(ret < 0) goto ABORT;
+			ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_NUM);
+			if(ret < 0) return -1;
 		}
-		ret = BitStream_appendNum(entry->bstream, MQRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
+		if(ret < 0) return -1;
 	} else {
-		ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_NUM);
-		if(ret < 0) goto ABORT;
-	
-		ret = BitStream_appendNum(entry->bstream, QRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_NUM);
+		if(ret < 0) return -1;
+
+		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_NUM, version), entry->size);
+		if(ret < 0) return -1;
 	}
 
 	words = entry->size / 3;
-	for(i=0; i<words; i++) {
+	for(i = 0; i < words; i++) {
 		val  = (entry->data[i*3  ] - '0') * 100;
 		val += (entry->data[i*3+1] - '0') * 10;
 		val += (entry->data[i*3+2] - '0');
 
-		ret = BitStream_appendNum(entry->bstream, 10, val);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 10, val);
+		if(ret < 0) return -1;
 	}
 
 	if(entry->size - words * 3 == 1) {
 		val = entry->data[words*3] - '0';
-		ret = BitStream_appendNum(entry->bstream, 4, val);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 4, val);
+		if(ret < 0) return -1;
 	} else if(entry->size - words * 3 == 2) {
 		val  = (entry->data[words*3  ] - '0') * 10;
 		val += (entry->data[words*3+1] - '0');
-		BitStream_appendNum(entry->bstream, 7, val);
-		if(ret < 0) goto ABORT;
+		BitStream_appendNum(bstream, 7, val);
+		if(ret < 0) return -1;
 	}
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
@@ -483,7 +477,7 @@ static int QRinput_checkModeAn(int size, const char *data)
 {
 	int i;
 
-	for(i=0; i<size; i++) {
+	for(i = 0; i < size; i++) {
 		if(QRinput_lookAnTable(data[i]) < 0)
 			return -1;
 	}
@@ -492,7 +486,7 @@ static int QRinput_checkModeAn(int size, const char *data)
 }
 
 /**
- * Estimates the length of the encoded bit stream of alphabet-numeric data.
+ * Estimate the length of the encoded bit stream of alphabet-numeric data.
  * @param size
  * @return number of bits
  */
@@ -511,7 +505,7 @@ int QRinput_estimateBitsModeAn(int size)
 }
 
 /**
- * Convert the alphabet-numeric data to a bit stream.
+ * Convert the alphabet-numeric data and append to a bit stream.
  * @param entry
  * @param mqr
  * @retval 0 success
@@ -520,51 +514,44 @@ int QRinput_estimateBitsModeAn(int size)
  * @throw ENOMEM unable to allocate memory.
  * @throw EINVAL invalid version.
  */
-static int QRinput_encodeModeAn(QRinput_List *entry, int version, int mqr)
+static int QRinput_encodeModeAn(QRinput_List *entry, BitStream *bstream, int version, int mqr)
 {
 	int words, i, ret;
 	unsigned int val;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
-
 	if(mqr) {
 		if(version < 2) {
 			errno = EINVAL;
-			goto ABORT;
+			return -1;
 		}
-		ret = BitStream_appendNum(entry->bstream, version - 1, MQRSPEC_MODEID_AN);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, MQRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_AN);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
+		if(ret < 0) return -1;
 	} else {
-		ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_AN);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, QRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_AN);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_AN, version), entry->size);
+		if(ret < 0) return -1;
 	}
 
 	words = entry->size / 2;
-	for(i=0; i<words; i++) {
+	for(i = 0; i < words; i++) {
 		val  = (unsigned int)QRinput_lookAnTable(entry->data[i*2  ]) * 45;
 		val += (unsigned int)QRinput_lookAnTable(entry->data[i*2+1]);
 
-		ret = BitStream_appendNum(entry->bstream, 11, val);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 11, val);
+		if(ret < 0) return -1;
 	}
 
 	if(entry->size & 1) {
 		val = (unsigned int)QRinput_lookAnTable(entry->data[words * 2]);
 
-		ret = BitStream_appendNum(entry->bstream, 6, val);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 6, val);
+		if(ret < 0) return -1;
 	}
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
@@ -572,7 +559,7 @@ ABORT:
  *****************************************************************************/
 
 /**
- * Estimates the length of the encoded bit stream of 8 bit data.
+ * Estimate the length of the encoded bit stream of 8 bit data.
  * @param size
  * @return number of bits
  */
@@ -582,7 +569,7 @@ int QRinput_estimateBitsMode8(int size)
 }
 
 /**
- * Convert the 8bits data to a bit stream.
+ * Convert the 8bits data and append to a bit stream.
  * @param entry
  * @param mqr
  * @retval 0 success
@@ -590,37 +577,30 @@ int QRinput_estimateBitsMode8(int size)
  *            See Execptions for the details.
  * @throw ENOMEM unable to allocate memory.
  */
-static int QRinput_encodeMode8(QRinput_List *entry, int version, int mqr)
+static int QRinput_encodeMode8(QRinput_List *entry, BitStream *bstream, int version, int mqr)
 {
 	int ret;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
-
 	if(mqr) {
 		if(version < 3) {
 			errno = EINVAL;
-			goto ABORT;
+			return -1;
 		}
-		ret = BitStream_appendNum(entry->bstream, version - 1, MQRSPEC_MODEID_8);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, MQRspec_lengthIndicator(QR_MODE_8, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_8);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_8, version), entry->size);
+		if(ret < 0) return -1;
 	} else {
-		ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_8);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, QRspec_lengthIndicator(QR_MODE_8, version), entry->size);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_8);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_8, version), entry->size);
+		if(ret < 0) return -1;
 	}
 
-	ret = BitStream_appendBytes(entry->bstream, entry->size, entry->data);
-	if(ret < 0) goto ABORT;
+	ret = BitStream_appendBytes(bstream, entry->size, entry->data);
+	if(ret < 0) return -1;
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 
@@ -629,7 +609,7 @@ ABORT:
  *****************************************************************************/
 
 /**
- * Estimates the length of the encoded bit stream of kanji data.
+ * Estimate the length of the encoded bit stream of kanji data.
  * @param size
  * @return number of bits
  */
@@ -652,7 +632,7 @@ static int QRinput_checkModeKanji(int size, const unsigned char *data)
 	if(size & 1)
 		return -1;
 
-	for(i=0; i<size; i+=2) {
+	for(i = 0; i < size; i+=2) {
 		val = ((unsigned int)data[i] << 8) | data[i+1];
 		if(val < 0x8140 || (val > 0x9ffc && val < 0xe040) || val > 0xebbf) {
 			return -1;
@@ -663,7 +643,7 @@ static int QRinput_checkModeKanji(int size, const unsigned char *data)
 }
 
 /**
- * Convert the kanji data to a bit stream.
+ * Convert the kanji data and append to a bit stream.
  * @param entry
  * @param mqr
  * @retval 0 success
@@ -672,31 +652,28 @@ static int QRinput_checkModeKanji(int size, const unsigned char *data)
  * @throw ENOMEM unable to allocate memory.
  * @throw EINVAL invalid version.
  */
-static int QRinput_encodeModeKanji(QRinput_List *entry, int version, int mqr)
+static int QRinput_encodeModeKanji(QRinput_List *entry, BitStream *bstream, int version, int mqr)
 {
 	int ret, i;
 	unsigned int val, h;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
-
 	if(mqr) {
 		if(version < 2) {
 			errno = EINVAL;
-			goto ABORT;
+			return -1;
 		}
-		ret = BitStream_appendNum(entry->bstream, version - 1, MQRSPEC_MODEID_KANJI);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, MQRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, version - 1, MQRSPEC_MODEID_KANJI);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, MQRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
+		if(ret < 0) return -1;
 	} else {
-		ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_KANJI);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_appendNum(entry->bstream, QRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_KANJI);
+		if(ret < 0) return -1;
+		ret = BitStream_appendNum(bstream, QRspec_lengthIndicator(QR_MODE_KANJI, version), entry->size/2);
+		if(ret < 0) return -1;
 	}
 
-	for(i=0; i<entry->size; i+=2) {
+	for(i = 0; i < entry->size; i+=2) {
 		val = ((unsigned int)entry->data[i] << 8) | entry->data[i+1];
 		if(val <= 0x9ffc) {
 			val -= 0x8140;
@@ -706,15 +683,11 @@ static int QRinput_encodeModeKanji(QRinput_List *entry, int version, int mqr)
 		h = (val >> 8) * 0xc0;
 		val = (val & 0xff) + h;
 
-		ret = BitStream_appendNum(entry->bstream, 13, val);
-		if(ret < 0) goto ABORT;
+		ret = BitStream_appendNum(bstream, 13, val);
+		if(ret < 0) return -1;
 	}
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
@@ -722,7 +695,7 @@ ABORT:
  *****************************************************************************/
 
 /**
- * Convert a structure symbol code to a bit stream.
+ * Convert a structure symbol code and append to a bit stream.
  * @param entry
  * @param mqr
  * @retval 0 success
@@ -731,7 +704,7 @@ ABORT:
  * @throw ENOMEM unable to allocate memory.
  * @throw EINVAL invalid entry.
  */
-static int QRinput_encodeModeStructure(QRinput_List *entry, int mqr)
+static int QRinput_encodeModeStructure(QRinput_List *entry, BitStream *bstream, int mqr)
 {
 	int ret;
 
@@ -739,54 +712,43 @@ static int QRinput_encodeModeStructure(QRinput_List *entry, int mqr)
 		errno = EINVAL;
 		return -1;
 	}
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
 
-	ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_STRUCTURE);
-	if(ret < 0) goto ABORT;
-	ret = BitStream_appendNum(entry->bstream, 4, entry->data[1] - 1);
-	if(ret < 0) goto ABORT;
-	ret = BitStream_appendNum(entry->bstream, 4, entry->data[0] - 1);
-	if(ret < 0) goto ABORT;
-	ret = BitStream_appendNum(entry->bstream, 8, entry->data[2]);
-	if(ret < 0) goto ABORT;
+	ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_STRUCTURE);
+	if(ret < 0) return -1;
+	ret = BitStream_appendNum(bstream, 4, entry->data[1] - 1);
+	if(ret < 0) return -1;
+	ret = BitStream_appendNum(bstream, 4, entry->data[0] - 1);
+	if(ret < 0) return -1;
+	ret = BitStream_appendNum(bstream, 8, entry->data[2]);
+	if(ret < 0) return -1;
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
  * FNC1
  *****************************************************************************/
 
-static int QRinput_checkModeFNC1Second(int size, const unsigned char *data)
+static int QRinput_checkModeFNC1Second(int size)
 {
 	if(size != 1) return -1;
 
+	/* No data check required. */
+
 	return 0;
 }
 
-static int QRinput_encodeModeFNC1Second(QRinput_List *entry, int version)
+static int QRinput_encodeModeFNC1Second(QRinput_List *entry, BitStream *bstream)
 {
 	int ret;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
+	ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_FNC1SECOND);
+	if(ret < 0) return -1;
 
-	ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_FNC1SECOND);
-	if(ret < 0) goto ABORT;
-	
-	ret = BitStream_appendBytes(entry->bstream, 1, entry->data);
-	if(ret < 0) goto ABORT;
+	ret = BitStream_appendBytes(bstream, 1, entry->data);
+	if(ret < 0) return -1;
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
@@ -798,7 +760,7 @@ static unsigned int QRinput_decodeECIfromByteArray(unsigned char *data)
 	unsigned int ecinum;
 
 	ecinum = 0;
-	for(i=0; i<4; i++) {
+	for(i = 0; i < 4; i++) {
 		ecinum = ecinum << 8;
 		ecinum |= data[3-i];
 	}
@@ -806,11 +768,11 @@ static unsigned int QRinput_decodeECIfromByteArray(unsigned char *data)
 	return ecinum;
 }
 
-int QRinput_estimateBitsModeECI(unsigned char *data)
+static int QRinput_estimateBitsModeECI(unsigned char *data)
 {
 	unsigned int ecinum;
 
-	ecinum = QRinput_decodeECIfromByteArray(data);;
+	ecinum = QRinput_decodeECIfromByteArray(data);
 
 	/* See Table 4 of JISX 0510:2004 pp.17. */
 	if(ecinum < 128) {
@@ -822,15 +784,12 @@ int QRinput_estimateBitsModeECI(unsigned char *data)
 	}
 }
 
-static int QRinput_encodeModeECI(QRinput_List *entry, int version)
+static int QRinput_encodeModeECI(QRinput_List *entry, BitStream *bstream)
 {
 	int ret, words;
 	unsigned int ecinum, code;
 
-	entry->bstream = BitStream_new();
-	if(entry->bstream == NULL) return -1;
-
-	ecinum = QRinput_decodeECIfromByteArray(entry->data);;
+	ecinum = QRinput_decodeECIfromByteArray(entry->data);
 
 	/* See Table 4 of JISX 0510:2004 pp.17. */
 	if(ecinum < 128) {
@@ -844,17 +803,13 @@ static int QRinput_encodeModeECI(QRinput_List *entry, int version)
 		code = 0xc0000 + ecinum;
 	}
 
-	ret = BitStream_appendNum(entry->bstream, 4, QRSPEC_MODEID_ECI);
-	if(ret < 0) goto ABORT;
-	
-	ret = BitStream_appendNum(entry->bstream, words * 8, code);
-	if(ret < 0) goto ABORT;
+	ret = BitStream_appendNum(bstream, 4, QRSPEC_MODEID_ECI);
+	if(ret < 0) return -1;
+
+	ret = BitStream_appendNum(bstream, words * 8, code);
+	if(ret < 0) return -1;
 
 	return 0;
-ABORT:
-	BitStream_free(entry->bstream);
-	entry->bstream = NULL;
-	return -1;
 }
 
 /******************************************************************************
@@ -881,7 +836,7 @@ int QRinput_check(QRencodeMode mode, int size, const unsigned char *data)
 		case QR_MODE_FNC1FIRST:
 			return 0;
 		case QR_MODE_FNC1SECOND:
-			return QRinput_checkModeFNC1Second(size, data);
+			return QRinput_checkModeFNC1Second(size);
 		case QR_MODE_NUL:
 			break;
 	}
@@ -894,7 +849,7 @@ int QRinput_check(QRencodeMode mode, int size, const unsigned char *data)
  *****************************************************************************/
 
 /**
- * Estimates the length of the encoded bit stream on the current version.
+ * Estimate the length of the encoded bit stream on the current version.
  * @param entry
  * @param version version of the symbol
  * @param mqr
@@ -950,12 +905,12 @@ static int QRinput_estimateBitStreamSizeOfEntry(QRinput_List *entry, int version
 }
 
 /**
- * Estimates the length of the encoded bit stream of the data.
+ * Estimate the length of the encoded bit stream of the data.
  * @param input input data
  * @param version version of the symbol
  * @return number of bits
  */
-__STATIC int QRinput_estimateBitStreamSize(QRinput *input, int version)
+STATIC_IN_RELEASE int QRinput_estimateBitStreamSize(QRinput *input, int version)
 {
 	QRinput_List *list;
 	int bits = 0;
@@ -970,7 +925,7 @@ __STATIC int QRinput_estimateBitStreamSize(QRinput *input, int version)
 }
 
 /**
- * Estimates the required version number of the symbol.
+ * Estimate the required version number of the symbol.
  * @param input input data
  * @return required version number
  */
@@ -984,22 +939,19 @@ static int QRinput_estimateVersion(QRinput *input)
 		prev = version;
 		bits = QRinput_estimateBitStreamSize(input, prev);
 		version = QRspec_getMinimumVersion((bits + 7) / 8, input->level);
-		if (version < 0) {
-			return -1;
-		}
 	} while (version > prev);
 
 	return version;
 }
 
 /**
- * Returns required length in bytes for specified mode, version and bits.
+ * Return required length in bytes for specified mode, version and bits.
  * @param mode
  * @param version
  * @param bits
  * @return required length of code words in bytes.
  */
-__STATIC int QRinput_lengthOfCode(QRencodeMode mode, int version, int bits)
+STATIC_IN_RELEASE int QRinput_lengthOfCode(QRencodeMode mode, int version, int bits)
 {
 	int payload, size, chunks, remain, maxsize;
 
@@ -1046,19 +998,18 @@ __STATIC int QRinput_lengthOfCode(QRencodeMode mode, int version, int bits)
  *****************************************************************************/
 
 /**
- * Convert the input data in the data chunk to a bit stream.
+ * Convert the input data in the data chunk and append to a bit stream.
  * @param entry
+ * @param bstream
  * @return number of bits (>0) or -1 for failure.
  */
-static int QRinput_encodeBitStream(QRinput_List *entry, int version, int mqr)
+static int QRinput_encodeBitStream(QRinput_List *entry, BitStream *bstream, int version, int mqr)
 {
 	int words, ret;
 	QRinput_List *st1 = NULL, *st2 = NULL;
+	int prevsize;
 
-	if(entry->bstream != NULL) {
-		BitStream_free(entry->bstream);
-		entry->bstream = NULL;
-	}
+	prevsize = BitStream_size(bstream);
 
 	words = QRspec_maximumWords(entry->mode, version);
 	if(words != 0 && entry->size > words) {
@@ -1067,41 +1018,36 @@ static int QRinput_encodeBitStream(QRinput_List *entry, int version, int mqr)
 		st2 = QRinput_List_newEntry(entry->mode, entry->size - words, &entry->data[words]);
 		if(st2 == NULL) goto ABORT;
 
-		ret = QRinput_encodeBitStream(st1, version, mqr);
+		ret = QRinput_encodeBitStream(st1, bstream, version, mqr);
 		if(ret < 0) goto ABORT;
-		ret = QRinput_encodeBitStream(st2, version, mqr);
-		if(ret < 0) goto ABORT;
-		entry->bstream = BitStream_new();
-		if(entry->bstream == NULL) goto ABORT;
-		ret = BitStream_append(entry->bstream, st1->bstream);
-		if(ret < 0) goto ABORT;
-		ret = BitStream_append(entry->bstream, st2->bstream);
+		ret = QRinput_encodeBitStream(st2, bstream, version, mqr);
 		if(ret < 0) goto ABORT;
+
 		QRinput_List_freeEntry(st1);
 		QRinput_List_freeEntry(st2);
 	} else {
 		ret = 0;
 		switch(entry->mode) {
 			case QR_MODE_NUM:
-				ret = QRinput_encodeModeNum(entry, version, mqr);
+				ret = QRinput_encodeModeNum(entry, bstream, version, mqr);
 				break;
 			case QR_MODE_AN:
-				ret = QRinput_encodeModeAn(entry, version, mqr);
+				ret = QRinput_encodeModeAn(entry, bstream, version, mqr);
 				break;
 			case QR_MODE_8:
-				ret = QRinput_encodeMode8(entry, version, mqr);
+				ret = QRinput_encodeMode8(entry, bstream, version, mqr);
 				break;
 			case QR_MODE_KANJI:
-				ret = QRinput_encodeModeKanji(entry, version, mqr);
+				ret = QRinput_encodeModeKanji(entry, bstream, version, mqr);
 				break;
 			case QR_MODE_STRUCTURE:
-				ret = QRinput_encodeModeStructure(entry, mqr);
+				ret = QRinput_encodeModeStructure(entry, bstream, mqr);
 				break;
 			case QR_MODE_ECI:
-				ret = QRinput_encodeModeECI(entry, version);
+				ret = QRinput_encodeModeECI(entry, bstream);
 				break;
 			case QR_MODE_FNC1SECOND:
-				ret = QRinput_encodeModeFNC1Second(entry, version);
+				ret = QRinput_encodeModeFNC1Second(entry, bstream);
 				break;
 			default:
 				break;
@@ -1109,7 +1055,7 @@ static int QRinput_encodeBitStream(QRinput_List *entry, int version, int mqr)
 		if(ret < 0) return -1;
 	}
 
-	return BitStream_size(entry->bstream);
+	return BitStream_size(bstream) - prevsize;
 ABORT:
 	QRinput_List_freeEntry(st1);
 	QRinput_List_freeEntry(st2);
@@ -1124,14 +1070,14 @@ ABORT:
  *            See Execptions for the details.
  * @throw ENOMEM unable to allocate memory.
  */
-static int QRinput_createBitStream(QRinput *input)
+static int QRinput_createBitStream(QRinput *input, BitStream *bstream)
 {
 	QRinput_List *list;
 	int bits, total = 0;
 
 	list = input->head;
 	while(list != NULL) {
-		bits = QRinput_encodeBitStream(list, input->version, input->mqr);
+		bits = QRinput_encodeBitStream(list, bstream, input->version, input->mqr);
 		if(bits < 0) return -1;
 		total += bits;
 		list = list->next;
@@ -1145,13 +1091,14 @@ static int QRinput_createBitStream(QRinput *input)
  * When the version number is given and that is not sufficient, it is increased
  * automatically.
  * @param input input data.
+ * @param bstream where the converted data is stored.
  * @retval 0 success
  * @retval -1 an error occurred and errno is set to indeicate the error.
  *            See Execptions for the details.
  * @throw ENOMEM unable to allocate memory.
- * @throw ERANGE input is too large.
+ * @throw ERANGE input data is too large.
  */
-static int QRinput_convertData(QRinput *input)
+static int QRinput_convertData(QRinput *input, BitStream *bstream)
 {
 	int bits;
 	int ver;
@@ -1162,13 +1109,11 @@ static int QRinput_convertData(QRinput *input)
 	}
 
 	for(;;) {
-		bits = QRinput_createBitStream(input);
+		BitStream_reset(bstream);
+		bits = QRinput_createBitStream(input, bstream);
 		if(bits < 0) return -1;
 		ver = QRspec_getMinimumVersion((bits + 7) / 8, input->level);
-		if(ver < 0) {
-			errno = ERANGE;
-			return -1;
-		} else if(ver > QRinput_getVersion(input)) {
+		if(ver > QRinput_getVersion(input)) {
 			QRinput_setVersion(input, ver);
 		} else {
 			break;
@@ -1191,8 +1136,6 @@ static int QRinput_convertData(QRinput *input)
 static int QRinput_appendPaddingBit(BitStream *bstream, QRinput *input)
 {
 	int bits, maxbits, words, maxwords, i, ret;
-	BitStream *padding = NULL;
-	unsigned char *padbuf;
 	int padlen;
 
 	bits = BitStream_size(bstream);
@@ -1208,39 +1151,25 @@ static int QRinput_appendPaddingBit(BitStream *bstream, QRinput *input)
 	}
 
 	if(maxbits - bits <= 4) {
-		ret = BitStream_appendNum(bstream, maxbits - bits, 0);
-		goto DONE;
+		return BitStream_appendNum(bstream, maxbits - bits, 0);
 	}
 
 	words = (bits + 4 + 7) / 8;
 
-	padding = BitStream_new();
-	if(padding == NULL) return -1;
-	ret = BitStream_appendNum(padding, words * 8 - bits, 0);
-	if(ret < 0) goto DONE;
+	ret = BitStream_appendNum(bstream, words * 8 - bits, 0);
+	if(ret < 0) return ret;
 
 	padlen = maxwords - words;
 	if(padlen > 0) {
-		padbuf = (unsigned char *)malloc(padlen);
-		if(padbuf == NULL) {
-			ret = -1;
-			goto DONE;
-		}
-		for(i=0; i<padlen; i++) {
-			padbuf[i] = (i&1)?0x11:0xec;
-		}
-		ret = BitStream_appendBytes(padding, padlen, padbuf);
-		free(padbuf);
-		if(ret < 0) {
-			goto DONE;
+		for(i = 0; i < padlen; i++) {
+			ret = BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
+			if(ret < 0) {
+				return ret;
+			}
 		}
 	}
 
-	ret = BitStream_append(bstream, padding);
-
-DONE:
-	BitStream_free(padding);
-	return ret;
+	return 0;
 }
 
 /**
@@ -1256,8 +1185,6 @@ DONE:
 static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 {
 	int bits, maxbits, words, maxwords, i, ret, termbits;
-	BitStream *padding = NULL;
-	unsigned char *padbuf;
 	int padlen;
 
 	bits = BitStream_size(bstream);
@@ -1275,8 +1202,7 @@ static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 	termbits = input->version * 2 + 1;
 
 	if(maxbits - bits <= termbits) {
-		ret = BitStream_appendNum(bstream, maxbits - bits, 0);
-		goto DONE;
+		return BitStream_appendNum(bstream, maxbits - bits, 0);
 	}
 
 	bits += termbits;
@@ -1288,38 +1214,23 @@ static int QRinput_appendPaddingBitMQR(BitStream *bstream, QRinput *input)
 	} else {
 		termbits += words * 8 - bits;
 	}
-	padding = BitStream_new();
-	if(padding == NULL) return -1;
-	ret = BitStream_appendNum(padding, termbits, 0);
-	if(ret < 0) goto DONE;
+	ret = BitStream_appendNum(bstream, termbits, 0);
+	if(ret < 0) return ret;
 
 	padlen = maxwords - words;
 	if(padlen > 0) {
-		padbuf = (unsigned char *)malloc(padlen);
-		if(padbuf == NULL) {
-			ret = -1;
-			goto DONE;
-		}
-		for(i=0; i<padlen; i++) {
-			padbuf[i] = (i&1)?0x11:0xec;
-		}
-		ret = BitStream_appendBytes(padding, padlen, padbuf);
-		free(padbuf);
-		if(ret < 0) {
-			goto DONE;
+		for(i = 0; i < padlen; i++) {
+			ret = BitStream_appendNum(bstream, 8, (i&1)?0x11:0xec);
+			if(ret < 0) return ret;
 		}
 		termbits = maxbits - maxwords * 8;
 		if(termbits > 0) {
-			ret = BitStream_appendNum(padding, termbits, 0);
-			if(ret < 0) goto DONE;
+			ret = BitStream_appendNum(bstream, termbits, 0);
+			if(ret < 0) return ret;
 		}
 	}
 
-	ret = BitStream_append(bstream, padding);
-
-DONE:
-	BitStream_free(padding);
-	return ret;
+	return 0;
 }
 
 static int QRinput_insertFNC1Header(QRinput *input)
@@ -1335,7 +1246,7 @@ static int QRinput_insertFNC1Header(QRinput *input)
 		return -1;
 	}
 
-	if(input->head->mode != QR_MODE_STRUCTURE || input->head->mode != QR_MODE_ECI) {
+	if(input->head->mode != QR_MODE_STRUCTURE && input->head->mode != QR_MODE_ECI) {
 		entry->next = input->head;
 		input->head = entry;
 	} else {
@@ -1352,41 +1263,24 @@ static int QRinput_insertFNC1Header(QRinput *input)
  * @return merged bit stream
  */
 
-__STATIC BitStream *QRinput_mergeBitStream(QRinput *input)
+STATIC_IN_RELEASE int QRinput_mergeBitStream(QRinput *input, BitStream *bstream)
 {
-	BitStream *bstream;
-	QRinput_List *list;
-	int ret;
-
 	if(input->mqr) {
-		if(QRinput_createBitStream(input) < 0) {
-			return NULL;
+		if(QRinput_createBitStream(input, bstream) < 0) {
+			return -1;
 		}
 	} else {
 		if(input->fnc1) {
 			if(QRinput_insertFNC1Header(input) < 0) {
-				return NULL;
+				return -1;
 			}
 		}
-		if(QRinput_convertData(input) < 0) {
-			return NULL;
+		if(QRinput_convertData(input, bstream) < 0) {
+			return -1;
 		}
 	}
 
-	bstream = BitStream_new();
-	if(bstream == NULL) return NULL;
-
-	list = input->head;
-	while(list != NULL) {
-		ret = BitStream_append(bstream, list->bstream);
-		if(ret < 0) {
-			BitStream_free(bstream);
-			return NULL;
-		}
-		list = list->next;
-	}
-
-	return bstream;
+	return 0;
 }
 
 /**
@@ -1395,26 +1289,21 @@ __STATIC BitStream *QRinput_mergeBitStream(QRinput *input)
  * @return padded merged bit stream
  */
 
-__STATIC BitStream *QRinput_getBitStream(QRinput *input)
+STATIC_IN_RELEASE int QRinput_getBitStream(QRinput *input, BitStream *bstream)
 {
-	BitStream *bstream;
 	int ret;
 
-	bstream = QRinput_mergeBitStream(input);
-	if(bstream == NULL) {
-		return NULL;
-	}
+	ret = QRinput_mergeBitStream(input, bstream);
+	if(ret < 0) return -1;
+
 	if(input->mqr) {
 		ret = QRinput_appendPaddingBitMQR(bstream, input);
 	} else {
 		ret = QRinput_appendPaddingBit(bstream, input);
 	}
-	if(ret < 0) {
-		BitStream_free(bstream);
-		return NULL;
-	}
+	if(ret < 0) return -1;
 
-	return bstream;
+	return 0;
 }
 
 /**
@@ -1427,11 +1316,18 @@ unsigned char *QRinput_getByteStream(QRinput *input)
 {
 	BitStream *bstream;
 	unsigned char *array;
+	int ret;
 
-	bstream = QRinput_getBitStream(input);
+	bstream = BitStream_new();
 	if(bstream == NULL) {
 		return NULL;
 	}
+
+	ret = QRinput_getBitStream(input, bstream);
+	if(ret < 0) {
+		BitStream_free(bstream);
+		return NULL;
+	}
 	array = BitStream_toByte(bstream);
 	BitStream_free(bstream);
 
@@ -1510,7 +1406,7 @@ int QRinput_Struct_appendInput(QRinput_Struct *s, QRinput *input)
 void QRinput_Struct_free(QRinput_Struct *s)
 {
 	QRinput_InputList *list, *next;
-	
+
 	if(s != NULL) {
 		list = s->head;
 		while(list != NULL) {
@@ -1553,7 +1449,7 @@ static int QRinput_List_shrinkEntry(QRinput_List *entry, int bytes)
 	return 0;
 }
 
-__STATIC int QRinput_splitEntry(QRinput_List *entry, int bytes)
+STATIC_IN_RELEASE int QRinput_splitEntry(QRinput_List *entry, int bytes)
 {
 	QRinput_List *e;
 	int ret;
@@ -1577,10 +1473,11 @@ __STATIC int QRinput_splitEntry(QRinput_List *entry, int bytes)
 
 QRinput_Struct *QRinput_splitQRinputToStruct(QRinput *input)
 {
-	QRinput *p;
-	QRinput_Struct *s;
+	QRinput *p = NULL;
+	QRinput_Struct *s = NULL;
 	int bits, maxbits, nextbits, bytes, ret;
 	QRinput_List *list, *next, *prev;
+	BitStream *bstream = NULL;
 
 	if(input->mqr) {
 		errno = EINVAL;
@@ -1599,11 +1496,10 @@ QRinput_Struct *QRinput_splitQRinputToStruct(QRinput *input)
 	QRinput_Struct_setParity(s, QRinput_calcParity(input));
 	maxbits = QRspec_getDataLength(input->version, input->level) * 8 - STRUCTURE_HEADER_SIZE;
 
-	if(maxbits <= 0) {
-		QRinput_Struct_free(s);
-		QRinput_free(input);
-		return NULL;
-	}
+	if(maxbits <= 0) goto ABORT;
+
+	bstream = BitStream_new();
+	if(bstream == NULL) goto ABORT;
 
 	bits = 0;
 	list = input->head;
@@ -1611,7 +1507,8 @@ QRinput_Struct *QRinput_splitQRinputToStruct(QRinput *input)
 	while(list != NULL) {
 		nextbits = QRinput_estimateBitStreamSizeOfEntry(list, input->version, input->mqr);
 		if(bits + nextbits <= maxbits) {
-			ret = QRinput_encodeBitStream(list, input->version, input->mqr);
+			BitStream_reset(bstream);
+			ret = QRinput_encodeBitStream(list, bstream, input->version, input->mqr);
 			if(ret < 0) goto ABORT;
 			bits += ret;
 			prev = list;
@@ -1657,19 +1554,23 @@ QRinput_Struct *QRinput_splitQRinputToStruct(QRinput *input)
 	ret = QRinput_Struct_appendInput(s, input);
 	if(ret < 0) goto ABORT;
 	if(s->size > MAX_STRUCTURED_SYMBOLS) {
-		QRinput_Struct_free(s);
 		errno = ERANGE;
+		QRinput_Struct_free(s);
+		BitStream_free(bstream);
 		return NULL;
 	}
 	ret = QRinput_Struct_insertStructuredAppendHeaders(s);
 	if(ret < 0) {
 		QRinput_Struct_free(s);
+		BitStream_free(bstream);
 		return NULL;
 	}
 
+	BitStream_free(bstream);
 	return s;
 
 ABORT:
+	BitStream_free(bstream);
 	QRinput_free(input);
 	QRinput_Struct_free(s);
 	return NULL;
@@ -1677,22 +1578,20 @@ ABORT:
 
 int QRinput_Struct_insertStructuredAppendHeaders(QRinput_Struct *s)
 {
-	int num, i;
+	int i;
 	QRinput_InputList *list;
 
+	if(s->size == 1) {
+		return 0;
+	}
+
 	if(s->parity < 0) {
 		QRinput_Struct_calcParity(s);
 	}
-	num = 0;
-	list = s->head;
-	while(list != NULL) {
-		num++;
-		list = list->next;
-	}
 	i = 1;
 	list = s->head;
 	while(list != NULL) {
-		if(QRinput_insertStructuredAppendHeader(list->input, num, i, s->parity))
+		if(QRinput_insertStructuredAppendHeader(list->input, s->size, i, s->parity))
 			return -1;
 		i++;
 		list = list->next;
