@@ -62,7 +62,7 @@ static int inputTest(QRinput_List *list, const char *fmt, ...)
 	return 0;
 }
 
-int inputSize(QRinput *input)
+static int inputSize(QRinput *input)
 {
 	BitStream *bstream;
 	int size;
@@ -75,7 +75,7 @@ int inputSize(QRinput *input)
 	return size;
 }
 
-void test_split1(void)
+static void test_split1(void)
 {
 	QRinput *input;
 	BitStream *bstream;
@@ -90,7 +90,7 @@ void test_split1(void)
 	BitStream_free(bstream);
 }
 
-void test_split2(void)
+static void test_split2(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -118,7 +118,7 @@ void test_split2(void)
 	QRinput_free(input);
 }
 
-void test_split3(void)
+static void test_split3(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -158,7 +158,7 @@ void test_split3(void)
 	QRinput_free(input);
 }
 
-void test_split4(void)
+static void test_split4(void)
 {
 	QRinput *input;
 	QRinput *i1, *i2;
@@ -219,7 +219,7 @@ void test_split4(void)
 	QRinput_free(i2);
 }
 
-void test_split5(void)
+static void test_split5(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -236,7 +236,7 @@ void test_split5(void)
 	QRinput_free(input);
 }
 
-void test_split6(void)
+static void test_split6(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -254,7 +254,7 @@ void test_split6(void)
 	QRinput_free(input);
 }
 
-void test_split7(void)
+static void test_split7(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -271,7 +271,7 @@ void test_split7(void)
 	QRinput_free(input);
 }
 
-void test_split8(void)
+static void test_split8(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -288,7 +288,7 @@ void test_split8(void)
 	QRinput_free(input);
 }
 
-void test_split3c(void)
+static void test_split3c(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -328,7 +328,7 @@ void test_split3c(void)
 	QRinput_free(input);
 }
 
-void test_toupper(void)
+static void test_toupper(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -378,7 +378,7 @@ void test_toupper(void)
 	QRinput_free(input);
 }
 
-void test_splitNum8(void)
+static void test_splitNum8(void)
 {
 	QRinput *input;
 	QRinput_List *list;
@@ -396,7 +396,7 @@ void test_splitNum8(void)
 	QRinput_free(input);
 }
 
-void test_splitAnNAn(void)
+static void test_splitAnNAn(void)
 {
 	QRinput *input1, *input2, *input3;
 	int s1, s2, s3;
@@ -429,7 +429,7 @@ void test_splitAnNAn(void)
 	QRinput_free(input3);
 }
 
-void test_splitAn8An(void)
+static void test_splitAn8An(void)
 {
 	QRinput *input1, *input2, *input3;
 	int s1, s2, s3;
@@ -462,7 +462,7 @@ void test_splitAn8An(void)
 	QRinput_free(input3);
 }
 
-void test_split8An8(void)
+static void test_split8An8(void)
 {
 	QRinput *input1, *input2, *input3;
 	int s1, s2, s3;
@@ -495,7 +495,7 @@ void test_split8An8(void)
 	QRinput_free(input3);
 }
 
-void test_split8N8(void)
+static void test_split8N8(void)
 {
 	QRinput *input1, *input2, *input3;
 	int s1, s2, s3;
@@ -528,7 +528,7 @@ void test_split8N8(void)
 	QRinput_free(input3);
 }
 
-int main(int argc, char **argv)
+int main()
 {
 	test_split1();
 	test_split2();
